SystemOrganization addCategory: #TusLibros!

Object subclass: #Cashier
	instanceVariableNames: 'paymentTerminal catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:33'!
emptyCartCheckoutMessage
	
	^ 'Cannot checkout empty carts.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:43'!
newWithCatalog: aCatalog andPaymentTerminal: aPaymentTerminal 
	
	^ self new initializeWithCatalog: aCatalog andPaymentTerminal: aPaymentTerminal.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:42'!
checkout: aCart usingCard: aCard
	| amountToDebit |
	self validateEmptyCart: aCart.
	
	amountToDebit := 0.
	aCart do: [ :anItem | amountToDebit := amountToDebit + (catalog at: anItem) ].
	paymentTerminal debit: amountToDebit from: aCard.! !

!Cashier methodsFor: 'initialize-release' stamp: 'JuanEdi 6/1/2013 22:47'!
initializeWithCatalog: aCatalog andPaymentTerminal: aPaymentTerminal
	
	catalog := aCatalog.
	paymentTerminal := aPaymentTerminal.! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 5/29/2013 21:26'!
validateEmptyCart: aShoppingCart 
	aShoppingCart isEmpty ifTrue: [self error: Cashier emptyCartCheckoutMessage]! !

Object subclass: #PaymentTerminalStub
	instanceVariableNames: 'transactionCount lastCardUsed lastAmountProcessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:03'!
debit: anAmount from: aCard 
	transactionCount := transactionCount + 1.
	lastCardUsed  := aCard.
	lastAmountProcessed := anAmount.! !

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:17'!
initialize

	transactionCount := 0.! !

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:36'!
lastAmountProcessed
	
	^ lastAmountProcessed! !

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:02'!
lastCardUsed
	
	^ lastCardUsed.! !

!PaymentTerminalStub methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 21:16'!
transactionCount
	
	^ transactionCount! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:23'!
invalidAmountOfProductsMessage
	
	^ 'Amount of products to add should be greater than zero'.! !

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'exception messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/1/2013 21:38'!
add: aProduct 
	self add: aProduct withOcurrences: 1.! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/1/2013 22:27'!
add: aProduct withOcurrences: aNumber 
	
	self validatePossitiveAmountOfProducts: aNumber.
	self validateCatalogContainsProduct: aProduct.
		
	products add: aProduct withOccurrences: aNumber.! !

!ShoppingCart methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:54'!
do: aBlock
	"Itera sobre cada ocurrencia individual de cada producto del carro."

	products do: aBlock! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 6/1/2013 22:24'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Bag new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:31'!
itemCount
	"Retorna la cantidad de items en total, contando todas las unidades de cada item agregado."
	^ products size.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 6/1/2013 22:55'!
occurrencesOf: aProduct 
	
	^ products occurrencesOf: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 6/1/2013 22:34'!
validateCatalogContainsProduct: aProduct

	(catalog keys includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:22'!
validatePossitiveAmountOfProducts: anAmount

	(anAmount > 0) ifFalse: [ self error: ShoppingCart invalidAmountOfProductsMessage]! !

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/1/2013 22:12'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 6/1/2013 22:54'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: catalog) isEmpty.! !

!ShoppingCartTest methodsFor: '2-Checkout' stamp: 'JuanEdi 6/1/2013 22:44'!
testShouldAddTransactionWhenCheckoutValidCart
	
	| cart cashier paymentTerminal initialTransactionCount |
	
	paymentTerminal := PaymentTerminalStub new.
	cashier := Cashier newWithCatalog: catalog andPaymentTerminal: paymentTerminal.
	
	initialTransactionCount := paymentTerminal transactionCount.
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter'.
	
	cashier checkout: cart usingCard: 'card12345'.
	
	self assert: paymentTerminal transactionCount = (initialTransactionCount + 1).! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 6/1/2013 22:50'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOcurrences: 2.
	self assert: (cart occurrencesOf: 'Dracula') = 2.
	self assert: (cart itemCount) = 2.! !

!ShoppingCartTest methodsFor: '2-Checkout' stamp: 'JuanEdi 6/1/2013 22:49'!
testShouldDebitTheCartAmountWhenCheckout
	| cart cashier paymentTerminal expectedAmount |
	
	paymentTerminal := PaymentTerminalStub new.
	cashier := Cashier newWithCatalog: catalog andPaymentTerminal: paymentTerminal.

	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter' withOcurrences: 2.
	cart add: 'Dracula' withOcurrences: 3.
	
	cashier checkout: cart usingCard: 'card12345'.
	
	expectedAmount := (2 * (catalog at: 'Harry Potter')) +  (3 * catalog at: 'Dracula').
	
	self assert: (paymentTerminal lastCardUsed) = 'card12345'.
	self assert: (paymentTerminal lastAmountProcessed) = expectedAmount.! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 6/1/2013 22:51'!
testShouldFailIfTryingToAddANegativeAmountOfProducts

	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter' withOcurrences: 2.
	self should: [ cart add: 'Harry Potter' withOcurrences: -1 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart invalidAmountOfProductsMessage ].
	self assert: (cart occurrencesOf: 'Harry Potter') = 2.
	self assert: (cart itemCount) = 2.
	! !

!ShoppingCartTest methodsFor: '2-Checkout' stamp: 'JuanEdi 6/1/2013 22:44'!
testShouldFailIfTryingToCheckoutEmptyCart
	
	| cashier paymentTerminal initialTransactionCount |
	
	paymentTerminal := PaymentTerminalStub new.
	cashier := Cashier newWithCatalog: catalog andPaymentTerminal: paymentTerminal.
	
	initialTransactionCount := paymentTerminal transactionCount.
	
	self should: [ cashier checkout: (ShoppingCart newWithCatalog: catalog) usingCard: '12345' ]
		  raise: Error
		  withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier emptyCartCheckoutMessage. 
			self assert: paymentTerminal transactionCount = initialTransactionCount.
		  ]! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 6/1/2013 22:52'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	self should: [ cart add: 'Cien aos de soledad' withOcurrences: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: '1-Shopping cart' stamp: 'JuanEdi 6/1/2013 22:52'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOcurrences: 2 .
	cart add: 'Dracula' withOcurrences: 3.
	self assert: (cart occurrencesOf: 'Dracula') = 5.
	self assert: (cart itemCount) = 5.! !
